@@ -14,7 +14,6 @@
 #include <QDebug>
 #include <QFile>
 #include <QGuiApplication>
-#include <memory> // std::unique_ptr
 
 int main(int argc, char *argv[])
 {
@@ -31,7 +30,7 @@ int main(int argc, char *argv[])
         return 2;
     }
 
-    // Note: If you update this code, update main.cpp as well.
+    // Note: If you update this code, update kcm.cpp as well.
 
     CursorTheme::applyCursorTheme(theme, size.toInt());
 
